# --Description: Create crossfades between tracks using AISAC

import cri.atomcraft.project
import cri.atomcraft.project as acproject
import cri.atomcraft.debug as acdebug

# --BeginUserVariable
"""
AISAC_CONTROL:
  type:object
  comment:AISAC-Control to use for the crossfades.
"""
AISAC_CONTROL = None
# --EndUserVariable

# Check if AISAC_CONTROL is specified and if it's type_name is correct.
if not AISAC_CONTROL:
	acdebug.warning("Specify a valid AISAC-Control in the script's User Variables to run the script.")
	sys.exit()

user_variable_type = acproject.get_type_name(AISAC_CONTROL)["data"]
if "AisacControl" not in user_variable_type:
	acdebug.warning(f"The User Variable type is [{user_variable_type}], please assign an AISAC-Control to run the script.") 
	sys.exit()

# Get selected tracks:
selected_tracks = acproject.get_selected_objects("Track")["data"]
if not selected_tracks:
	acdebug.warning("No track selected. Please select two or more tracks.")
	sys.exit()

# Get number of tracks
track_number = len(selected_tracks)

# Check if track number > 1
if track_number < 2:
	acdebug.warning("Please select two or more tracks.")
	sys.exit()

# Get range between points
point_rng = 1 / (track_number - 1)

# Loop through the selected tracks to create Volume graphs
for i in range(track_number):
	# Define AISAC graph name
	aisac_graph_name = f"AISAC Crossfade {i + 1}"

	# Create AISAC under the current track:
	track_aisac = acproject.create_aisac(selected_tracks[i], aisac_graph_name, AISAC_CONTROL)["data"]

	# Create Volume graph:
	volume_graph = acproject.create_aisac_graph(track_aisac, "Volume")["data"]

	# Create graph points based on the position in the track list
	# Create a start point, except for the first track
	if i > 0:
		control_value_start = (point_rng * (i - 1))
		destination_value_start = 0
		graph_point_start = acproject.create_aisac_point(volume_graph, control_value_start, destination_value_start)["data"]

	# Create a center point
	control_value_center = (point_rng * i)
	destination_value_center = 1
	graph_point_center = acproject.create_aisac_point(volume_graph, control_value_center, destination_value_center)["data"]

	# Change curve type of the center point, except for the first track
	if i > 0:
		curvetype_center = "FastChange"
		acproject.set_value(graph_point_center, "CurveType", curvetype_center)

	# Create an end point, except for the last track 
	if i < track_number - 1:
		control_value_end = (point_rng * (i + 1))
		destination_value_end = 0
		graph_point_end = acproject.create_aisac_point(volume_graph, control_value_end, destination_value_end)["data"]

		# Change curve type of the end point
		curvetype_end = "SlowChange"
		acproject.set_value(graph_point_end, "CurveType", curvetype_end)

	# Log message
	track_name = acproject.get_value(selected_tracks[i], "Name")["data"]
	acdebug.log(f"AISAC graph [{aisac_graph_name}] has been created under [{track_name}].")